/*----------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2004 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
----------------------------------------------------------------------*/


/*----------------------------------------------------------------------
 Disable "unreferenced formal parameter" compiler warning
----------------------------------------------------------------------*/
#ifndef _DEBUG
	#pragma warning ( disable : 4100 )
#endif

/*----------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
----------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes--------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
/*---Shell Includes---------------------------------------------------*/
#include "lsapi.h"
#include "../sdk/include/purels.h"
/*--------------------------------------------------------------------*/


typedef struct _DIALOGHWNDITEM
{
	HWND hDlgWnd;
	//
	struct _DIALOGHWNDITEM * pNext;
	//
} DIALOGHWNDITEM, *PDIALOGHWNDITEM;


/*---Global Variables-------------------------------------------------*/
static PDIALOGHWNDITEM s_pDlgList;
/*--------------------------------------------------------------------*/


/*--------------------------------------------------------------------*\
------------------------- Function Definitions -------------------------
\*--------------------------------------------------------------------*/


/*--------IsModuleDialogMsg---------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
BOOL IsModuleDialogMsg(LPMSG lpMsg)
{
	PDIALOGHWNDITEM pdhi = s_pDlgList;

	while(NULL != pdhi)
	{
		if(IsDialogMessage(pdhi->hDlgWnd, lpMsg))
		{
			return(TRUE);
		}

		pdhi = pdhi->pNext;
	}

	return(FALSE);
}

/*--------RegisterDialogWnd---------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
VOID RegisterDialogWnd(HWND hDlgWnd)
{
	PDIALOGHWNDITEM pdhi;

	pdhi = (PDIALOGHWNDITEM)malloc(sizeof(DIALOGHWNDITEM));

	pdhi->hDlgWnd = hDlgWnd;

	pdhi->pNext = s_pDlgList;
	s_pDlgList = pdhi;

	return;
}

/*--------UnregisterDialogWnd-------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
VOID UnregisterDialogWnd(HWND hDlgWnd)
{
	PDIALOGHWNDITEM pdhi = s_pDlgList, pprev = NULL;

	while(NULL != pdhi)
	{
		if(hDlgWnd == pdhi->hDlgWnd)
		{
			if( NULL == pprev )
			{
				s_pDlgList = pdhi->pNext;
			}
			else
			{
				pprev->pNext = pdhi->pNext;
			}

			free(pdhi);
			break;
		}

		pprev = pdhi;
		pdhi = pdhi->pNext;
	}

	return;
}

/*--------RemoveModuleDialogs-------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
VOID RemoveModuleDialogs(VOID)
{
	PDIALOGHWNDITEM pdhi;

	while(NULL != s_pDlgList)
	{
		pdhi = s_pDlgList->pNext;

		free(s_pDlgList);

		s_pDlgList = pdhi;
	}

	return;
}

/*--------CleanDeadDialogs----------------------------------------------
 Purpose:
 - 
----------------------------------------------------------------------*/
VOID CleanDeadDialogs(VOID)
{
	PDIALOGHWNDITEM pdhi = s_pDlgList, pprev = NULL, pnext;

	while(NULL != pdhi)
	{
		pnext = pdhi->pNext;

		if(!IsWindow(pdhi->hDlgWnd))
		{
			if(NULL == pprev)
			{
				s_pDlgList = pnext;
			}
			else
			{
				pprev->pNext = pnext;
			}

			free(pdhi);
		}
		else
		{
			pprev = pdhi;
		}

		pdhi = pnext;
	}

	return;
}

/*--------End Of File-------------------------------------------------*/
