/*

  This is a part of the LiteStep Shell Source code modified for use with
	PureLS Source code.

  Copyright (C) 2000 - 2004 The PureLS Development Team
  Copyright (C) 1997-98 The LiteStep Development Team

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

	/* Disable "unreferenced formal parameter" compiler warning. */
#ifndef _DEBUG
	#pragma warning ( disable : 4100 )
#endif

	/* Lets just make sure we aren't depending on IE */
#define _WIN32_IE 0x0000

/*** System Includes ***********/
#include <tchar.h>
#include <windows.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>
/*******************************/

/*** Shell Includes ************/
#include "lsapi.h"
#include "../sdk/include/purels.h"
/*******************************/

extern HWND g_hMainWnd;
extern LPTSTR g_pszAppPath;
extern MS_RunFunc fpMSRun;
extern MS_WinShutdownFunc fpMSWinShutdown;


typedef struct
{
	LPTSTR name;
	FP_BANG_COMMAND func;
	FP_BANG_COMMAND_EX funcEx;
} tBangCommand;

static tBangCommand *s_pDynamicBangs;
static UINT s_uDynamicBangCount;

static const tBangCommand s_pInternalBangs[] = \
{
		/* System Commands */
	{ _T("!LOGOFF"),   BangLogoff,   NULL },
	{ _T("!RUN"),      BangRun,      NULL },
	{ _T("!SHUTDOWN"), BangShutdown, NULL },
	{ _T("!STARTUP"),  BangStartup,  NULL },

		/* Window Managing Commands */
	{ _T("!CASCADEWINDOWS"), BangCascadeWindows, NULL },
	{ _T("!TILEWINDOWSH"),   BangTileWindowsH,   NULL },
	{ _T("!TILEWINDOWSV"),   BangTileWindowsV,   NULL },

		/* Shell Commands */
	{ _T("!ABOUT"),	        BangAbout,         NULL },
	{ _T("!EXECUTE"),       BangExecute,       NULL },
	{ _T("!QUIT"),          BangQuit,          NULL },
	{ _T("!RECYCLE"),       BangRecycle,       NULL },
	{ _T("!REFRESH"),       BangRefresh,       NULL },
	{ _T("!LOADMODULE"),    BangLoadModule,    NULL },
	{ _T("!UNLOADMODULE"),  BangUnloadModule,  NULL },
	{ _T("!RELOADMODULE"),  BangReloadModule,  NULL },
	{ _T("!REGISTERVAR"),   BangRegisterVar,   NULL },
	{ _T("!UNREGISTERVAR"), BangUnregisterVar, NULL }
};

#define INTERNAL_BANG_COUNT ((UINT)(sizeof(s_pInternalBangs) / sizeof(tBangCommand)))


/******************************************\
* Internal Bang Command Support Functions
\******************************************/
BOOL GetInternalBC( LPCTSTR pszBangCmd, UINT *puIndex )
{
	if(NULL != pszBangCmd)
	{
		UINT uIndex;

		for(uIndex = 0; uIndex < INTERNAL_BANG_COUNT; uIndex++)
		{
			if(0 == _tcsicmp(s_pInternalBangs[uIndex].name, pszBangCmd))
			{
				if(NULL != puIndex)
				{
					*puIndex = uIndex;
				}

				return TRUE;
			}
		}
	}

	return FALSE;
}

BOOL GetDynamicBC( LPCTSTR pszBangCmd, UINT *puIndex )
{
	if(NULL != pszBangCmd && NULL != s_pDynamicBangs)
	{
		UINT uIndex;

		for(uIndex = 0; uIndex < s_uDynamicBangCount; uIndex++)
		{
			if(0 == _tcsicmp(s_pDynamicBangs[uIndex].name, pszBangCmd))
			{
				if(NULL != puIndex)
				{
					*puIndex = uIndex;
				}

				return TRUE;
			}
		}
	}

	return FALSE;
}

VOID UpdateDynamicBC( UINT uIndex, FP_BANG_COMMAND fpBangCmd)
{
	s_pDynamicBangs[uIndex].func = fpBangCmd;
	s_pDynamicBangs[uIndex].funcEx = NULL;

	return;
}

VOID UpdateDynamicBCEx( UINT uIndex, FP_BANG_COMMAND_EX fpBangCmdEx)
{
	s_pDynamicBangs[uIndex].func = NULL;
	s_pDynamicBangs[uIndex].funcEx = fpBangCmdEx;

	return;
}

BOOL CreateBC( LPCTSTR pszBangCmd, UINT *puIndex )
{
	BOOL bRetVal = FALSE;

	/* Only proceed if passed in params are valid */
	if(NULL != pszBangCmd && _T('!') == *pszBangCmd)
	{
		LPTSTR pszTemp;

		pszTemp = (LPTSTR)malloc((_tcslen(pszBangCmd) + 1) * sizeof(_TCHAR));

		if(NULL != pszTemp)
		{
				/* Store old memory block pointer */
			tBangCommand *pOldBangs = (tBangCommand*)s_pDynamicBangs;

			_tcscpy(pszTemp, pszBangCmd);

				/* Allocate a new block of memory */
			if (s_pDynamicBangs == NULL)
			{
				s_uDynamicBangCount = 0;
				s_pDynamicBangs = (tBangCommand *)malloc(sizeof(tBangCommand));
			}
			else
			{
				s_pDynamicBangs = (tBangCommand *)realloc(
					 s_pDynamicBangs
					,(s_uDynamicBangCount + 1) * sizeof(tBangCommand)
				);
			}

			if (s_pDynamicBangs == NULL)
			{
					/* Memory allocation failed, restore old valid pointer and exit */
				s_pDynamicBangs = pOldBangs;
				free(pszTemp);
			}
			else
			{
				/* Memory allocated worked, lets populate and set our pass flag */
				s_pDynamicBangs[s_uDynamicBangCount].func = NULL;
				s_pDynamicBangs[s_uDynamicBangCount].funcEx = NULL;
				s_pDynamicBangs[s_uDynamicBangCount].name = pszTemp;

				if(NULL != puIndex)
				{
					*puIndex = s_uDynamicBangCount;
				}

				s_uDynamicBangCount++;

				bRetVal = TRUE;
			}
		}
	}

	return bRetVal;
}

VOID PurgeBCs( VOID )
{
	UINT uIndex;

	for(uIndex = 0; uIndex < s_uDynamicBangCount; uIndex++)
	{
		free(s_pDynamicBangs[uIndex].name);
	}

	s_uDynamicBangCount = 0;

	if(NULL != s_pDynamicBangs)
	{
		free(s_pDynamicBangs);
		s_pDynamicBangs = NULL;
	}

	return;
}


/******************************************\
* Exported Bang Command Functions
\******************************************/
BOOL IsBangCommand ( LPCTSTR pszBangCmd )
{
	return( GetDynamicBC( pszBangCmd, NULL ) || GetInternalBC( pszBangCmd, NULL ) );
}

BOOL AddBangCommand( LPCTSTR pszBangCmd, FP_BANG_COMMAND fpBangCmd )
{
	UINT nBCIndex;
	BOOL bRetVal = FALSE;

	if(GetDynamicBC(pszBangCmd, &nBCIndex) || CreateBC(pszBangCmd, &nBCIndex))
	{
		UpdateDynamicBC(nBCIndex, fpBangCmd);
		bRetVal = TRUE;
	}

	return(bRetVal);
}

BOOL AddBangCommandEx( LPCTSTR pszBangCmd, FP_BANG_COMMAND_EX pszBangCmdEx )
{
	UINT nBCIndex;
	BOOL bRetVal = FALSE;

	if(GetDynamicBC(pszBangCmd, &nBCIndex) || CreateBC(pszBangCmd, &nBCIndex))
	{
		UpdateDynamicBCEx(nBCIndex, pszBangCmdEx);
		bRetVal = TRUE;
	}

	return(bRetVal);
}

BOOL RemoveBangCommand( LPCTSTR pszBangCmd )
{
	UINT uIndex;
	BOOL bRetVal = FALSE;

	if (GetDynamicBC(pszBangCmd, &uIndex))
	{
		free(s_pDynamicBangs[uIndex].name);

		if (uIndex < --s_uDynamicBangCount)
		{
			memmove(
				 &s_pDynamicBangs[uIndex]
				,&s_pDynamicBangs[uIndex+1]
				,(s_uDynamicBangCount - uIndex) * sizeof(tBangCommand)
			);
		}

		if (s_uDynamicBangCount > 0)
		{
			tBangCommand *pOldBangs = s_pDynamicBangs;

			s_pDynamicBangs = (tBangCommand*)realloc(
				 s_pDynamicBangs
				,s_uDynamicBangCount * sizeof(tBangCommand)
			);

			if (s_pDynamicBangs == NULL && pOldBangs != NULL)
			{
				s_pDynamicBangs = pOldBangs;

				memset(s_pDynamicBangs + s_uDynamicBangCount, 0, sizeof(tBangCommand));
			}
		}
		else if (s_pDynamicBangs != NULL)
		{
			free(s_pDynamicBangs);
			s_pDynamicBangs = NULL;
		}

		bRetVal = TRUE;
	}

	return bRetVal;
}

BOOL ParseBangCommand( HWND hCallerWnd, LPCTSTR pszBangCmd, LPCTSTR pszArgs )
{
	BOOL bRetVal = FALSE;

	if (pszBangCmd != NULL && *pszBangCmd == _T('!'))
	{
		_TCHAR szBangCmd[PLS_MAX_SIZE];
		_TCHAR szArgs[PLS_MAX_SIZE];
		UINT uIndex;

		_tcscpy(szBangCmd, pszBangCmd);

		*szArgs = 0;
		if (pszArgs != NULL && *pszArgs != 0)
		{
			_tcscpy(szArgs, pszArgs);
		}

		if (GetDynamicBC(pszBangCmd, &uIndex))
		{
			if (NULL != s_pDynamicBangs[uIndex].funcEx)
			{
				s_pDynamicBangs[uIndex].funcEx(hCallerWnd, szBangCmd, szArgs);
			}
			else
			{
				s_pDynamicBangs[uIndex].func(hCallerWnd, szArgs);
			}

			bRetVal = TRUE;
		}
		else if (GetInternalBC(pszBangCmd, &uIndex))
		{
			if (NULL != s_pInternalBangs[uIndex].funcEx)
			{
				s_pInternalBangs[uIndex].funcEx(hCallerWnd, szBangCmd, szArgs);
			}
			else
			{
				s_pInternalBangs[uIndex].func(hCallerWnd, szArgs);
			}

			bRetVal = TRUE;
		}
	}

	return bRetVal;
}


/******************************************\
* Internal Bang Commands
\******************************************/
VOID __cdecl BangLogoff( HWND hCallerWnd, LPCTSTR pszArgs )
{
	if (g_hMainWnd)
	{
		PostMessage (g_hMainWnd, PM_QUIT, QC_LOGOFF, 0);
	}

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
	UNREFERENCED_PARAMETER(pszArgs);
}

VOID __cdecl BangRun( HWND hCallerWnd, LPCTSTR pszArgs )
{
	if(NULL != fpMSRun)
	{
		fpMSRun(NULL, 0, 0, NULL, NULL, 0);
	}

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
	UNREFERENCED_PARAMETER(pszArgs);
}

VOID __cdecl BangShutdown( HWND hCallerWnd, LPCTSTR pszArgs )
{
	if(NULL != fpMSWinShutdown)
	{
		fpMSWinShutdown(g_hMainWnd);
	}

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
	UNREFERENCED_PARAMETER(pszArgs);
}

VOID __cdecl BangStartup( HWND hCallerWnd, LPCTSTR pszArgs )
{
	_TCHAR tmpPath[1024] = {0};
	SS_Startup fpStartup;
	HANDLE hShellsvc;

	_stprintf(tmpPath, _T("%s\\%s"), g_pszAppPath, _T("shellsvc.dll"));

	hShellsvc = LoadLibrary(tmpPath);

	if(NULL != hShellsvc)
	{
		fpStartup = (SS_Startup)GetProcAddress(
			 hShellsvc
			,"SS_ForceStartup"
		);

		if(NULL != fpStartup)
		{
			HANDLE hStartupThread;
			UINT uDummy;

			hStartupThread = (HANDLE)_beginthreadex(
				 NULL
				,0
				,fpStartup
				,NULL
				,0
				,&uDummy
			);

			if(NULL != hStartupThread)
			{
				while(WAIT_OBJECT_0 != WaitForSingleObject(hStartupThread, 10000))
				{
					if(IDYES == WarningMessageBox(
						 _T("The !Startup command has not finished,\n")
						 _T("would you like to terminate the procedure?\n\n")
						 _T("It is not recommended to terminate !Startup.\n")
						 _T("Press NO to wait another 30 seconds for !Startup to complete.")
						,_T("PureLS - Terminate !Startup?")
						,MB_YESNO
					))
					{
						if(WAIT_OBJECT_0 != WaitForSingleObject(hStartupThread, 1000))
						{
							TerminateThread(hStartupThread, (DWORD)-1);
						}
					}
				}

				CloseHandle(hStartupThread);
			}
		}

		FreeLibrary(hShellsvc);
	}

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
	UNREFERENCED_PARAMETER(pszArgs);
}

VOID __cdecl BangCascadeWindows( HWND hCallerWnd, LPCTSTR pszArgs )
{
	CascadeWindows(NULL, MDITILE_SKIPDISABLED, NULL, 0, NULL);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
	UNREFERENCED_PARAMETER(pszArgs);
}

VOID __cdecl BangTileWindowsH( HWND hCallerWnd, LPCTSTR pszArgs )
{
	TileWindows(NULL, MDITILE_HORIZONTAL, NULL, 0, NULL);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
	UNREFERENCED_PARAMETER(pszArgs);
}

VOID __cdecl BangTileWindowsV( HWND hCallerWnd, LPCTSTR pszArgs )
{
	TileWindows(NULL, MDITILE_VERTICAL, NULL, 0, NULL);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
	UNREFERENCED_PARAMETER(pszArgs);
}

VOID __cdecl BangAbout( HWND hCallerWnd, LPCTSTR pszArgs )
{
	LPARAM lParam;

	lParam = AD_TYPE_ACTIVE|AD_TYPE_TOPMOST;

	if(NULL != pszArgs && 0 != *pszArgs)
	{
		if(0 == _tcsicmp(pszArgs, _T("detailed")) )
		{
			lParam |= AD_TYPE_DETAIL;
		}
	}

	SendMessage(g_hMainWnd, LSAPI_ABOUTDIALOG, AD_CMD_OPEN, lParam);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
}

VOID __cdecl BangExecute( HWND hCallerWnd, LPCTSTR pszArgs )
{
	_TCHAR buf1[PLS_MAX_SIZE];
	_TCHAR buf2[PLS_MAX_SIZE];
	BOOL bGotToken = FALSE;
	HINSTANCE err = 0;

	_tcscpy(buf1, pszArgs);

	while (PL_BreakTokens(buf2, buf1, buf1))
	{
		if (*buf2 == 0)
		{
			continue;
		}

		bGotToken = TRUE;
		err = LSExecute(hCallerWnd, buf2, 0);
	}

	if (!bGotToken && *buf1 != 0)
	{
		err = LSExecute(hCallerWnd, buf1, 0);
	}

	if ((HINSTANCE)ERROR_FILE_NOT_FOUND == err)
	{
		MessageBoxPrintf(
			 MB_OK|MB_ICONERROR|MB_TASKMODAL|MB_TOPMOST|MB_SETFOREGROUND
			,_T("PureLS - Error!")
			,_T("!Execute:\n\n\"%s\"\n\nInvalid command or file does not exist!")
			,bGotToken ? buf2 : buf1
		);
	}

	return;
}

VOID __cdecl BangQuit( HWND hCallerWnd, LPCTSTR pszArgs )
{
	if (g_hMainWnd)
	{
		PostMessage (g_hMainWnd, PM_QUIT, 0, 0);
	}

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
	UNREFERENCED_PARAMETER(pszArgs);
}

VOID __cdecl BangRecycle( HWND hCallerWnd, LPCTSTR pszArgs )
{
	PostMessage(g_hMainWnd, PM_RECYCLE, 0, 0);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
	UNREFERENCED_PARAMETER(pszArgs);
}

VOID __cdecl BangRefresh( HWND hCallerWnd, LPCTSTR pszArgs )
{
	PostMessage(g_hMainWnd, PM_REFRESH, 0, 0);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
	UNREFERENCED_PARAMETER(pszArgs);
}

VOID __cdecl BangLoadModule( HWND hCallerWnd, LPCTSTR pszArgs )
{
	StartModule(pszArgs);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
}

VOID __cdecl BangUnloadModule( HWND hCallerWnd, LPCTSTR pszArgs )
{
	StopModule(pszArgs);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
}

VOID __cdecl BangReloadModule( HWND hCallerWnd, LPCTSTR pszArgs )
{
	ReloadModule(pszArgs);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
}

VOID __cdecl BangRegisterVar( HWND hCallerWnd, LPCTSTR pszArgs )
{
	SH_CFG_RegisterVar(pszArgs);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
}

VOID __cdecl BangUnregisterVar( HWND hCallerWnd, LPCTSTR pszArgs )
{
	SH_CFG_RegisterVar(pszArgs);

	return;
	UNREFERENCED_PARAMETER(hCallerWnd);
}


