 /*

  This is a part of the LiteStep Shell Source code modified for use with
  PureLS Source code.

  Copyright (C) 2000 - 2004 The PureLS Development Team
  Copyright (C) 1997-98 The LiteStep Development Team

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

	/* Disable "unreferenced formal parameter" compiler warning. */
#ifndef _DEBUG
	#pragma warning ( disable : 4100 )
#endif

	/* Lets just make sure we aren't depending on IE */
#define _WIN32_IE 0x0000

/*** System Includes ***********/
#include <tchar.h>
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <shellapi.h>
/*******************************/

/*** Shell Includes ************/
#include "lsapi.h"
#include "../sdk/include/purels.h"
/*******************************/


extern HWND g_hMainWnd;
extern LPTSTR g_pszAppPath;


HINSTANCE LSExecuteEx( HWND Owner, LPCTSTR szOperation, LPCTSTR szCommand, LPCTSTR szArgs, LPCTSTR szDirectory, INT nShowCmd )
{
	HINSTANCE hRetVal = (HINSTANCE)ERROR_FILE_NOT_FOUND;

	if (*szCommand == _T('!'))
	{
		if (ParseBangCommand(Owner, (LPTSTR)szCommand, (LPTSTR)szArgs))
		{
			hRetVal = (HINSTANCE)33;
		}
	}
	else
	{
		SHELLEXECUTEINFO si;
		SetLastError(0);

		memset(&si, 0, sizeof(SHELLEXECUTEINFO));
		si.cbSize = sizeof(SHELLEXECUTEINFO);
		si.hwnd = Owner;
		si.fMask = SEE_MASK_DOENVSUBST|SEE_MASK_FLAG_NO_UI;
		si.lpVerb = szOperation;
		si.lpFile = szCommand;
		si.lpParameters = szArgs;
		si.lpDirectory = szDirectory;
		si.nShow = nShowCmd;
		ShellExecuteEx(&si);
		hRetVal = si.hInstApp;
	}

	return hRetVal;
}

HINSTANCE LSExecute(HWND Owner, LPCTSTR szCommand, INT nShowCmd)
{
	HINSTANCE retVal = 0;

	if (szCommand != NULL && *szCommand != 0)
	{
		_TCHAR buf1[PLS_MAX_SIZE];
		_TCHAR buf2[PLS_MAX_SIZE];

		if (PL_SimpleToken(buf1, buf2, SH_CFG_ExpandBuffer(buf1, szCommand, PLS_MAX_SIZE)))
		{
			if (*buf2 == 0 && (_tcschr(buf1, _T('\"')) || _tcschr(buf1, _T('\''))))
			{
				PL_SimpleToken(buf1, buf2, buf1);
			}

			if(*buf1 == _T('!'))
			{
				retVal = LSExecuteEx(Owner, NULL, buf1, buf2, NULL, 0);
			}
			else
			{
				_TCHAR folder[_MAX_DIR], full_directory[_MAX_DIR + _MAX_DRIVE + 1];
				_tsplitpath(buf1, full_directory, folder, NULL, NULL);
				_tcscat(full_directory, folder);

				retVal = LSExecuteEx(
					Owner,
					NULL,
					buf1,
					buf2,
					full_directory,
					nShowCmd ? nShowCmd:SW_SHOWNORMAL
				);
			}
		}
	}
	return retVal;
}

VOID SetDesktopArea(INT left, INT top, INT right, INT bottom)
{
	RECT r;

	r.left = left;
	r.top = top;
	r.right = right;
	r.bottom = bottom;
	SystemParametersInfo(SPI_SETWORKAREA,0,(PVOID)&r,SPIF_SENDCHANGE);
	SystemParametersInfo(SPI_GETWORKAREA,0,(PVOID)&r,SPIF_SENDCHANGE);
	return;
}

HWND GetLitestepWnd( VOID )
{
	return g_hMainWnd;
}

VOID GetResStr(HINSTANCE hInstance, UINT uIDText, LPTSTR pszText, INT nLen, LPCTSTR pszDefText)
{
	if(NULL != hInstance && NULL != pszText && nLen > 0)
	{
		*pszText = 0;

		if(0 == LoadString(hInstance, uIDText, pszText, nLen) && NULL != pszDefText)
		{
			_tcsncpy(pszText, pszDefText, nLen);
			pszText[nLen - 1] = 0;
		}
	}

	return;
}

VOID GetResStrEx(HINSTANCE hInstance, UINT uIDText, LPTSTR pszText, INT nLen, LPCTSTR pszDefText, ...)
{
	if(NULL != hInstance && NULL != pszText && nLen > 0)
	{
		_TCHAR szFormat[PLS_MAX_SIZE];
		_TCHAR szBuffer[PLS_MAX_SIZE];
		va_list vargs;

		GetResStr(hInstance, uIDText, szFormat, PLS_MAX_SIZE, pszDefText);

		va_start(vargs, pszDefText);
		_vstprintf(szBuffer, szFormat, vargs);
		va_end(vargs);

		_tcsncpy(pszText, szBuffer, nLen);
		pszText[nLen - 1] = 0;
	}

	return;
}

BOOL WINAPI LSGetLitestepPath( LPTSTR pszPath, size_t nMaxLen )
{
	if(NULL != pszPath && NULL != g_pszAppPath && _tcslen(g_pszAppPath)+2 <= nMaxLen)
	{
		*pszPath = 0;
		_tcscpy(pszPath, g_pszAppPath);
		_tcscat(pszPath, _T("\\")); /* Litestep modules expect trailing '\' */

		return(TRUE);
	}

	return(FALSE);
}

BOOL WINAPI LSGetImagePath( LPTSTR pszPath, size_t nMaxLen )
{
	if(NULL != pszPath)
	{
		TCHAR szPath[MAX_PATH] = {0};
		UINT nLen;

		*pszPath = 0;

		if(!SH_CFG_GetString(szPath, _T("LSImageFolder"), NULL, MAX_PATH))
		{
			if(!SH_CFG_GetString(szPath, _T("PixmapPath"), NULL, MAX_PATH))
			{
				if(LSGetLitestepPath(szPath, MAX_PATH - 10))
				{
					_tcscat(szPath, _T("IMAGES\\"));
				}
			}
		}

		nLen = _tcslen(szPath);

		if(0 != nLen)
		{
			if(szPath[nLen - 1] != _T('\\'))
			{
				_tcscat(szPath, _T("\\")); /* Litestep modules expect trailing '\' */
				nLen++;
			}

			if(nLen < nMaxLen)
			{
				_tcscpy(pszPath, szPath);
				return(TRUE);
			}
			//_tcsncpy(pszPath, szPath, nMaxLen);
			//pszPath[nMaxLen - 1] = 0;
			//return(TRUE);
		}
	}

	return(FALSE);
}

/*
	The rest of these functions are only implemented because people refuse to
	quit using them. So, please quit using them!
*/
BOOL WINAPI LSLog(INT nLevel, LPCTSTR pszModule, LPCTSTR pszMessage)
{
	return(TRUE);
	UNREFERENCED_PARAMETER(nLevel);
	UNREFERENCED_PARAMETER(pszModule);
	UNREFERENCED_PARAMETER(pszMessage);
}

BOOL WINAPIV LSLogPrintf(INT nLevel, LPCTSTR pszModule, LPCTSTR pszMessage, ...)
{
	return(TRUE);
	UNREFERENCED_PARAMETER(nLevel);
	UNREFERENCED_PARAMETER(pszModule);
	UNREFERENCED_PARAMETER(pszMessage);
}

INT ParseCoordinate(LPCTSTR szString, INT nDefault, INT nMaxVal)
{
	BOOL negative = FALSE;
	BOOL center = FALSE;
	BOOL percentual = FALSE;

	INT value = 0;

	size_t length = _tcslen(szString);
	size_t i = 0;

	if(0 == *szString)
	{
		return(nDefault);
	}

	if(_T('-') == *szString)
	{
		negative = TRUE;
		szString++;
		length--;
	}
	else if(_T('+') == *szString)
	{
		szString++;
		length--;
	}

	while(i < length)
	{
		if (szString[i] >= _T('0') && szString[i] <= _T('9'))
		{
			value = (value * 10) + (szString[i] - _T('0'));
		}
		else
		{
			break;
		}

		++i;
	}

	if(i == 0)
	{
		return(nDefault);
	}

	while(i < length)
	{
		if(!center && _T('c') == szString[i] || _T('C') == szString[i])
		{
			center = TRUE;
		}
		else if (!percentual && _T('%') == szString[i])
		{
			percentual = TRUE;
		}
		else
		{
			break;
		}

		++i;
	}

	if(percentual)
	{
		value = nMaxVal * value / 100;
	}

	if(center)
	{
		if(negative)
		{
			value = (nMaxVal / 2) - value;
		}
		else
		{
			value = (nMaxVal / 2) + value;
		}
	}
	else if(negative)
	{
		value = nMaxVal - value;
	}

	return(value);
}

INT GetRCCoordinate(LPCTSTR pszKeyName, INT nDefault, INT nMaxVal)
{
	_TCHAR strVal[PLS_MAX_SIZE];

	if(!GetRCString(pszKeyName, strVal, NULL, PLS_MAX_SIZE))
	{
		return(nDefault);
	}

	return(ParseCoordinate(strVal, nDefault, nMaxVal));
}

