/*------------------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation; either version 2 of the License, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
------------------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes----------------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
/*---Shell Includes-----------------------------------------------------------*/
#include "../sdk/include/plsdefs.h"
#include "main.h"
/*----------------------------------------------------------------------------*/


/*---Global Variables---------------------------------------------------------*/
static HINSTANCE g_hMainInst;
static HANDLE g_hShellMutex;

HANDLE g_hLitestepThread;
HANDLE g_hServicesThread;
HANDLE g_hStartupThread;

HANDLE g_hShellReadyEvent;

DWORD g_dwStartup;
DWORD g_dwInstance;

BOOL g_bNT5Tray;
BOOL g_bServices;

LPTSTR g_pszAppPath;
LPTSTR g_pszCfgFile;
/*----------------------------------------------------------------------------*/




/*--------GetHomePath-----------------------------------------------------------
 Purpose:
 - To get our local path, and point g_pszAppPath at the allocated string
------------------------------------------------------------------------------*/
LPCTSTR GetHomePath(
	VOID
)
{
	/*--------------------------------------------------------------------------
	 No need to reallocate it if we already have done it

	 NOTE: This check is not necessary as we know we only call this one time.
	 However, it is not a big overhead, so we will leave it
	--------------------------------------------------------------------------*/
	if(NULL == g_pszAppPath)
	{
		_TCHAR tmpAppPath[MAX_PATH];
		DWORD len;

		/*----------------------------------------------------------------------
		 GetModuleFileName includes both the path and file name and we only want
		 the path.  So, truncate it until we see the trailing '\' character.
		----------------------------------------------------------------------*/
		len = GetModuleFileName(NULL, tmpAppPath, MAX_PATH);
		while(len && tmpAppPath[--len] != _T('\\'));

		/*----------------------------------------------------------------------
		 malloc is not called if "len" evaluates to zero
		----------------------------------------------------------------------*/
		if(
			0 != len
			&&
			NULL != (g_pszAppPath = (LPTSTR)malloc((len+1)*sizeof(_TCHAR)))
		)
		{
			/*------------------------------------------------------------------
			 Copy over the correct number of characters, and then zero terminate
			 the buffer.
			------------------------------------------------------------------*/
			_tcsncpy(g_pszAppPath, tmpAppPath, len);
			g_pszAppPath[len] = 0;
		}
	}

	return(g_pszAppPath);
}


/*--------FileExists------------------------------------------------------------
 Purpose:
 - Verifies specified path is a valid file
------------------------------------------------------------------------------*/
BOOL FileExists(
	LPCTSTR const szFilename
)
{
	/*
	return(
		0xFFFFFFFF != GetFileAttributes(szFilename)
		&&
		!(
			GetFileAttributes(szFilename)
			&
			FILE_ATTRIBUTE_DIRECTORY
		)
	)
	*/
	UINT uErrMode;
	HANDLE hFile;
	BOOL bRet;

	uErrMode = SetErrorMode(SEM_FAILCRITICALERRORS);

	hFile = CreateFile(
		 szFilename
		,0
		,FILE_SHARE_READ
		,NULL
		,OPEN_EXISTING
		,FILE_ATTRIBUTE_NORMAL
		,NULL
	);
 
	bRet = (INVALID_HANDLE_VALUE != hFile);

	if(bRet)
	{
		CloseHandle(hFile);
	}

	SetErrorMode(uErrMode);

	return(bRet);
}


/*--------ParseCommandLine------------------------------------------------------
 Purpose:
 - Parses command line and configures global settings from parsed values.
------------------------------------------------------------------------------*/
BOOL ParseCommandLine(
	LPCTSTR const pCmdLine
)
{
	_TCHAR temp[MAX_PATH] = {0};
	BOOL bRet = TRUE;
	LPTSTR pStr;
	LPTSTR ptr;
	LPTSTR ptmp;

	/*--------------------------------------------------------------------------
	 Setup default configuration settings
	--------------------------------------------------------------------------*/
	g_bNT5Tray = TRUE;
	g_bServices = TRUE;

	g_dwInstance = dINSTANCE_NORMAL;
	g_dwStartup = dSTARTUP_NORMAL;

	/*--------------------------------------------------------------------------
	 Parse the command line
	--------------------------------------------------------------------------*/
	pStr = (LPTSTR)malloc((_tcslen(pCmdLine) + 1) * sizeof( _TCHAR ));

	if( NULL != pStr )
	{
		ptr = _tcslwr(_tcscpy(pStr, pCmdLine));
	}
	else
	{
		ptr = temp;
	}

	while( 0 != *ptr )
	{
		/*----------------------------------------------------------------------
		 Parse string if *ptr is a '-'
		----------------------------------------------------------------------*/
		if( _T('-') == *ptr++ && 0 != *ptr)
		{
			switch( *ptr++ )
			{
			/*------------------------------------------------------------------
			 -sX
			  0 - Disable startup items
			  1 - Force startup items

			 NOTE: If the SHIFT key was being held down, that overrides this
			 command line option.

			 EXAMPLE: -s0
			------------------------------------------------------------------*/
			case _T('s'):
				if(_istdigit(*ptr))
				{
					if(CMD_LINE__STARTUP_DISABLE == (_TCHAR)(*ptr - _T('0')))
					{
						g_dwStartup = dSTARTUP_DISABLE;
					}
					else if(CMD_LINE__STARTUP_FORCE == (_TCHAR)(*ptr - _T('0')))
					{
						g_dwStartup = dSTARTUP_FORCE;
					}
				}
				break;

			/*------------------------------------------------------------------
			 -mX
			  0 - Subsequent instances of PureLS are ended w/o warning
			  1 - Allow multiple instance of PureLS w/o notification

			 EXAMPLE: -m
			------------------------------------------------------------------*/
			case _T('m'):
				if(_istdigit(*ptr))
				{
					if(CMD_LINE__INSTANCE_SINGLE == (_TCHAR)(*ptr - _T('0')))
					{
						g_dwInstance = dINSTANCE_SINGLE;
					}
					else if(CMD_LINE__INSTANCE_MULTI == (_TCHAR)(*ptr - _T('0')))
					{
						g_dwInstance = dINSTANCE_MULTI;
					}
				}
				break;

			/*------------------------------------------------------------------
			 -t
			  Disable NT5 systray support

			 EXAMPLE: -t
			------------------------------------------------------------------*/
			case _T('t'):
				g_bNT5Tray = FALSE;
				break;

			/*------------------------------------------------------------------
			 -n
			  Disable all shell services (startup/systray/tasks)

			 EXAMPLE: -n
			------------------------------------------------------------------*/
			case _T('n'):
				g_bServices = FALSE;
				break;

#if 0
			case _T('!'): /* Bang command support or some such ? */
				ptmp = temp;

				/* Increment to next token, or end of cmdline */
				while( 0 != *ptr && _T('-') != *ptr )
				{
					*ptmp++ = *ptr++;
				}

				*ptmp = 0;

				while( _istspace(*(ptmp-1)) )
				{
					*(--ptmp) = 0;
				}
				break;
#endif

			/*------------------------------------------------------------------
			 -f<file>
			  Specifies path to purels config file. It defaults to "purels.cfg"
			  located in the same folder as purels.exe.

			 EXAMPLE: -fc:\shells\purels\themes\default\purels.cfg
			------------------------------------------------------------------*/
			case _T('f'):
				ptmp = temp;

				/* Increment to next token, or end of cmdline */
				while( 0 != *ptr && _T('-') != *ptr )
				{
					*ptmp++ = *ptr++;
				}

				*ptmp = 0;

				while( _istspace(*(ptmp-1)) )
				{
					*(--ptmp) = 0;
				}

				/*--------------------------------------------------------------
				 Check to see if specified file exists.
				--------------------------------------------------------------*/
				if( FileExists(temp) )
				{
					if( NULL == g_pszCfgFile )
					{
						g_pszCfgFile = (LPTSTR)malloc(
							(_tcslen(temp)+1)*sizeof(_TCHAR)
						);

						if( NULL != g_pszCfgFile )
						{
							_tcscpy(g_pszCfgFile, temp);
						}
					}
				}
				break;
			}
		}
	}

	if( NULL != pStr )
	{
		free(pStr);
	}

	return( bRet );
}

#ifdef _UNICODE
/*--------CheckSetup------------------------------------------------------------
 Purpose:
 - Determines whether or not we are running on an NT platform
 NOTE:
 - Use ANSI implementations so it can check on Win9x platforms even though we
   are buit for UNICODE.
------------------------------------------------------------------------------*/
BOOL IsPlatformNT( VOID )
{
	OSVERSIONINFOA osvi;

	memset(&osvi, 0 , sizeof(OSVERSIONINFOA));
	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFOA);

	GetVersionExA(&osvi);

	return(VER_PLATFORM_WIN32_NT == osvi.dwPlatformId);
}
#endif /* _UNICODE */

/*--------CheckSetup------------------------------------------------------------
 Purpose:
 - Stores path we are being run from.
 - Checks command line options
 - Verifies multiple instances are not running
------------------------------------------------------------------------------*/
BOOL CheckSetup(
	LPCTSTR const pCmdLine
)
{
	BOOL retVal = FALSE;

	/*--------------------------------------------------------------------------
	 - If we are a UNICODE build, verify we are on an NT platform

	 - Retrieve our path, and only continue if it was successfully determined
	   and allocated.

	 - If ParseCommandLine() returns FALSE, then do not continue to load.  This
	   may be because one of the command line options specifies not to continue
	   loading after parsing its value.
	--------------------------------------------------------------------------*/
	if(
#ifdef _UNICODE
		IsPlatformNT()
		&&
#endif /* _UNICODE */
		NULL != GetHomePath()
		&&
		ParseCommandLine(pCmdLine)
	)
	{
		/*----------------------------------------------------------------------
		 Create a named mutex, so that we can detect if we are already running.
		----------------------------------------------------------------------*/
		SetLastError(ERROR_SUCCESS);
		g_hShellMutex = CreateMutex(NULL, FALSE, _T("ShellMutex"));

		/*----------------------------------------------------------------------
		 Disallow multiple instances to run at once unless the user says "yes"
		 via the message box, or has requested multiple instances be allowed via
		 the command line options.
		----------------------------------------------------------------------*/
		if(NULL != g_hShellMutex)
		{
			/*------------------------------------------------------------------
			 Set retVal to TRUE if
			 - If g_dwInstance specifies allowing multiple instances to run.
			 or
			 - GetLastError() does not return ERROR_ALREADY_EXISTS
			 or
			 - g_dwInstance does not specify that only a single instance is
			   allowed, and the user chooses "yes" to allow multiple instances

			 NOTE: Currently Shell Services are still loaded each time an
			 instance is run.  Should we auto disable them when multiple
			 instance are running?
			------------------------------------------------------------------*/
			retVal = (
				dINSTANCE_MULTI == g_dwInstance
				||
				ERROR_ALREADY_EXISTS != GetLastError()
				||
				(
					dINSTANCE_SINGLE != g_dwInstance
					&&
					IDYES == WarningMessageBox(
						PLSTEXT_INSTANCE_MSG,
						PLSTEXT_INSTANCE_CAPTION,
						MB_YESNO
					)
				)
			);
		}
	}

	return( retVal );
}


/*--------FreeSetup-------------------------------------------------------------
 Purpose:
 - Frees all resources allocated from CheckSetup()
------------------------------------------------------------------------------*/
VOID FreeSetup(
	VOID
)
{
	if( NULL != g_pszCfgFile )
	{
		free(g_pszCfgFile);
		g_pszCfgFile = NULL;
	}

	if( NULL != g_pszAppPath )
	{
		free(g_pszAppPath);
		g_pszAppPath = NULL;
	}

	if( NULL != g_hShellMutex )
	{
		CloseHandle(g_hShellMutex);
		g_hShellMutex = NULL;
	}

	return;
}


/*--------EventHandler----------------------------------------------------------
 Purpose:
 - Handles all events (mainly thread exits)
 - Handles all messages posted to our thread.
------------------------------------------------------------------------------*/
VOID EventHandler(
	VOID
)
{
	HANDLE pObjects[NUM_WAIT_OBJECTS];
	BOOL bExit;
	DWORD dwRet;
	DWORD dwNumObj;
	MSG msg;

#ifndef _DEBUG
	/*--------------------------------------------------------------------------
	 Tells Windows the shell has finished loading

	 NOTE: We need to look into this further, and verify its use.
	--------------------------------------------------------------------------*/
	if( g_bServices )
	{
		SendMessage(GetDesktopWindow(),0x400,0,0);
	}
#endif /* not _DEBUG */

	bExit = FALSE;

	while( !bExit )
	{
		dwNumObj = 0;

		if( NULL != g_hLitestepThread )
		{
			*(pObjects + dwNumObj) = g_hLitestepThread;
			dwNumObj++;
		}

		if( NULL != g_hServicesThread )
		{
			*(pObjects + dwNumObj) = g_hServicesThread;
			dwNumObj++;
		}

		if( NULL != g_hStartupThread )
		{
			*(pObjects + dwNumObj) = g_hStartupThread;
			dwNumObj++;
		}

		/*----------------------------------------------------------------------
		 Wait until we have something to process
		----------------------------------------------------------------------*/
		dwRet = MsgWaitForMultipleObjects(
			 dwNumObj
			,pObjects
			,FALSE
			,INFINITE
			,QS_ALLEVENTS
		);

		/*----------------------------------------------------------------------
		 Check to see what our Wait returned
		 - WAIT_FAILED - Failure obviously
		 - WAIT_OBJECT (plus object array size) - message received
		 - WAIT_OBJECT (plus offset into object array) - object signaled
		----------------------------------------------------------------------*/
		if( (WAIT_OBJECT_0 + dwNumObj) == dwRet )
		{
			while( PeekMessage(&msg, INVALID_HANDLE_VALUE, 0, 0, PM_REMOVE) )
			{
				if( WM_QUIT == msg.message )
				{
					bExit = TRUE;
				}
			}
		}
		else if( dwRet >= WAIT_OBJECT_0 && dwRet < (WAIT_OBJECT_0 + dwNumObj) )
		{
			DWORD dwObj = dwRet - WAIT_OBJECT_0;

			if(
				NULL != g_hLitestepThread
				&&
				pObjects[dwObj] == g_hLitestepThread
			)
			{
				CloseHandle(g_hLitestepThread);
				g_hLitestepThread = NULL;

				PostQuitMessage(0);
			}
			else if(
				NULL != g_hServicesThread
				&&
				pObjects[dwObj] == g_hServicesThread
			)
			{
				CloseHandle(g_hServicesThread);
				g_hServicesThread = NULL;

				if(NULL == g_hStartupThread)
				{
					UnloadServices();
				}
			}
			else if(
				NULL != g_hStartupThread
				&&
				pObjects[dwObj] == g_hStartupThread
			)
			{
				CloseHandle(g_hStartupThread);
				g_hStartupThread = NULL;

				if(NULL == g_hServicesThread)
				{
					UnloadServices();
				}
			}
		}
	}

	return;
}


/*--------WinMain---------------------------------------------------------------
 Purpose:
 - Entry point into application
 - Calls all utility functions for startup and initialization
 - Calls EventHandler()
 - Calls all clean up functions and deinitializers
------------------------------------------------------------------------------*/
INT WINAPI _tWinMain(
	 HINSTANCE hInstance
	,HINSTANCE hPrevInstance
	,LPSTR pCmdLine
	,INT nCmdShow
)
{
	/*--------------------------------------------------------------------------
	 Notes:
	  hPrevInstance is always NULL
	  pCmdLine does not support wchar strings, so use GetCommandLine()
	--------------------------------------------------------------------------*/
	if( NULL == hPrevInstance && CheckSetup(GetCommandLine()) )
	{
		/*----------------------------------------------------------------------
		 Store our instance
		----------------------------------------------------------------------*/
		g_hMainInst = hInstance;

		/*----------------------------------------------------------------------
		 Create our "ready" event that releases shell initialization items.
		----------------------------------------------------------------------*/
		g_hShellReadyEvent = CreateEvent(
			 NULL
			,TRUE
			,FALSE
			,_T("ShellIsReadyEvent")
		);

		if( NULL != g_hShellReadyEvent )
		{
			/*------------------------------------------------------------------
			 Force Windows to immediately create a message queue for our thread
			 to be used in EventHandler().  Otherwise it is possible we will
			 miss some messages that are sent to us before we get into our
			 message loop.

			 NOTE: Windows creates a message queue for our thread at the first
			 call to a user/gdi function, so the following PostThreadMessage()
			 does the trick.
			------------------------------------------------------------------*/
			PostThreadMessage(GetCurrentThreadId(), WM_NULL, 0, 0);

			/*------------------------------------------------------------------
			 Load configuration buffer
			------------------------------------------------------------------*/
			LoadConfig();

			/*------------------------------------------------------------------
			 Load Shell Services (startup, systray, taskhook)
			------------------------------------------------------------------*/
			LoadServices();

			/*------------------------------------------------------------------
			 Load Litestep Module System thread
			------------------------------------------------------------------*/
			LoadLitestepModuleSystem();

			/*------------------------------------------------------------------
			 Set ShellReady event
			------------------------------------------------------------------*/
			SetEvent(g_hShellReadyEvent);

			/*------------------------------------------------------------------
			 Sit in here until we are ready to exit and clean up.
			------------------------------------------------------------------*/
			EventHandler();

			/*------------------------------------------------------------------
			 Unload our Litestep Module System - Likely already unloaded
			------------------------------------------------------------------*/
			UnloadLitestepModuleSystem();

			/*------------------------------------------------------------------
			 Unload our Shell Services
			------------------------------------------------------------------*/
			UnloadServices();

			/*------------------------------------------------------------------
			 Unload Config buffer
			------------------------------------------------------------------*/
			UnloadConfig();

			/*------------------------------------------------------------------
			 Close the handle to our shell ready event
			------------------------------------------------------------------*/
			CloseHandle(g_hShellReadyEvent);
			g_hShellReadyEvent = NULL;
		}

		/*----------------------------------------------------------------------
		 Clear our global storage of our instance handle
		----------------------------------------------------------------------*/
		g_hMainInst = NULL;
	}
	else
	{
		/*----------------------------------------------------------------------
		 Report to the user why we didn't load for a few error cases.

		 NOTE: I eventually want to implement a local Get/SetLastError and then
		 we can do a final 'switch' statement before exiting to check to see
		 what the error was and inform the user.  Otherwise we are doing
		 repetitive checks like this to reverify why we are exiting.
		----------------------------------------------------------------------*/
#ifdef _UNICODE
		if(!IsPlatformNT())
		{
			ErrorMessageBox(
				PLSTEXT_EXIT_UNICODE_MSG,
				PLSTEXT_EXIT_CAPTION,
				MB_OK
			);
		}
		else
#endif
		if(NULL == g_pszAppPath)
		{
			ErrorMessageBox(
				PLSTEXT_EXIT_PATH_MSG,
				PLSTEXT_EXIT_CAPTION,
				MB_OK
			);
		}
	}

	/*--------------------------------------------------------------------------
	 Release any memory allocation made while setting up our environment
	--------------------------------------------------------------------------*/
	FreeSetup();

	/*--------------------------------------------------------------------------
	 That's all folks!
	--------------------------------------------------------------------------*/
	return( 0 );

	UNREFERENCED_PARAMETER(nCmdShow);
	UNREFERENCED_PARAMETER(pCmdLine);
}


#if 0
/*
 IS THERE A WAY TO IMPLEMENT A DOS 'main' FUNCTION SO WE CAN RECEIVE COMMAND
 LINE OPTIONS FROM A DOS WINDOW?  AS WELL AS PRINT OUT WHAT THOSE OPTIONS ARE?
*/
VOID __cdecl _tmain ( INT argc, LPTSTR* argv )
{

// HANDLE DOS COMMAND LINE OPTIONS

	return;
}
#endif

/*--------End Of File---------------------------------------------------------*/
