/*------------------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation; either version 2 of the License, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
------------------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes----------------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <process.h>
#include <stdio.h>
#include <stdlib.h>
#include <shlobj.h>
/*---Shell Includes-----------------------------------------------------------*/
#include "services.h"
#include "../sdk/include/except.h"
#include "../sdk/include/plsdefs.h"
/*----------------------------------------------------------------------------*/


/*---Global Variables---------------------------------------------------------*/
static HMODULE s_hShellsvc;
static HMODULE s_hConfig;
static DWORD s_dwLSThread;
static DWORD s_dwSSThread;
static DWORD s_dwSUThread;
static BOOL s_bFailLoad;

//define_exception_type(INT);
//struct exception_context the_exception_context[1];
/*----------------------------------------------------------------------------*/


/*--------RunLitestepModuleSystem-----------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
UINT __stdcall RunLitestepModuleSystem(
	HANDLE hLitestepEvent
)
{
	HMODULE hLitestep;
	_TCHAR szPath[MAX_PATH];
	UINT uRet = (UINT)-1;

	_stprintf(szPath, _T("%s\\%s"), g_pszAppPath, _T("lsapi.dll"));
	hLitestep = LoadLibrary(szPath);

	if(NULL != hLitestep)
	{
		typedef INT (WINAPI *LS_LitestepMainFunc)(LPCTSTR, HANDLE);
		LS_LitestepMainFunc fpLitestepMain;

		fpLitestepMain = (LS_LitestepMainFunc)GetProcAddress(
			 hLitestep
			,"ShellMain"
		);

		if( NULL != fpLitestepMain )
		{
#ifdef PLS_EXCEPTION_HANDLER

			/*------------------------------------------------------------------
			 Before we actually run the litestep main, we will try to locate an
			 exception handler from the lsapi.dll as well.
			------------------------------------------------------------------*/
			PLS_ExceptionHandler fpLsExceptionHandler = \
				(PLS_ExceptionHandler)GetProcAddress(
					 hLitestep
					,"ShellExceptionHandler"
				);

#ifdef _DEBUG
			OutputDebugString(_T("[grd] RunLitestepModuleSystem()::"));
			OutputDebugString(_T("PLS_ExceptionHandlerPush\r\n"));
#endif

			if( NULL != fpLsExceptionHandler )
			{
				PLS_ExceptionHandlerPush(fpLsExceptionHandler);
			}

#endif /* PLS_EXCEPTION_HANDLER */

			/*------------------------------------------------------------------
			 Turn control over to lsapi.dll
			------------------------------------------------------------------*/
			uRet = (UINT)fpLitestepMain(g_pszAppPath, hLitestepEvent);

#ifdef PLS_EXCEPTION_HANDLER

			/*------------------------------------------------------------------
			 Make the litestep exception handler go out of scope.
			------------------------------------------------------------------*/
			if(NULL != fpLsExceptionHandler)
			{
				PLS_ExceptionHandlerPop();
			}

#endif /* PLS_EXCEPTION_HANDLER */
		}

		FreeLibrary(hLitestep);
		hLitestep = NULL;
	}

	if((UINT)-1 == uRet)
	{
		s_bFailLoad = TRUE;

		if(NULL != hLitestepEvent)
		{
			SetEvent(hLitestepEvent);
		}
	}

	_endthreadex(uRet);
	return(uRet);
}

/*--------RunShellServices------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
UINT __stdcall RunShellServices(
	HANDLE hServicesEvent
)
{
	SS_MainFunc fpMain;
	UINT uRet = (UINT)-1;

	fpMain = (SS_MainFunc)GetProcAddress(s_hShellsvc, "SS_Main");

	SetEvent(hServicesEvent);

	if(NULL != fpMain)
	{
		uRet = (UINT)fpMain(g_bNT5Tray);
	}

	_endthreadex(uRet);
	return(uRet);
}



/*****************/
/* wrapper calls */
/*****************/



/*--------UnloadLitestepModuleSystem--------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID UnloadLitestepModuleSystem(
	VOID
)
{
	/*--------------------------------------------------------------------------
	 Verify Litestep Module System Thread has finished
	--------------------------------------------------------------------------*/
	if(NULL != g_hLitestepThread)
	{
		DWORD dwRet;

		/*----------------------------------------------------------------------
		 Exit Litestep Module System thread
		----------------------------------------------------------------------*/
		PostThreadMessage(s_dwLSThread, PM_QUIT, 0, 0);

		/*----------------------------------------------------------------------
		 Allow for a long 10 second timeout before killing the thread. Hopefully
		 it is never actually necessary. Also, lets tell the user to report
		 this, and then check again to see if the thread finally ended so we
		 don't kill it unnecessarily.
		----------------------------------------------------------------------*/
		dwRet = WaitForSingleObject(g_hLitestepThread, 10000);

		if(WAIT_OBJECT_0 != dwRet)
		{
			WarningMessageBox(
				 PLSTEXT_TERMINATETHREAD_LITESTEP_MSG
				,PLSTEXT_TERMINATETHREAD_CAPTION
				,MB_OK
			);

			dwRet = WaitForSingleObject(g_hLitestepThread, 0);

			if(WAIT_OBJECT_0 != dwRet)
			{
				TerminateThread(g_hLitestepThread, (DWORD)-1);
			}
		}

		CloseHandle(g_hLitestepThread);
		g_hLitestepThread = NULL;
	}

	return;
}

/*--------LoadLitestepModuleSystem----------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID LoadLitestepModuleSystem(
	VOID
)
{
	HANDLE hLitestepEvent;

	hLitestepEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	if(NULL != hLitestepEvent)
	{
		UINT uDummy;

		g_hLitestepThread = (HANDLE)_beginthreadex(
			 NULL
			,0
			,RunLitestepModuleSystem
			,hLitestepEvent
			,0
			,&uDummy
		);

		if(NULL != g_hLitestepThread)
		{
			WaitForSingleObject(hLitestepEvent, INFINITE);
			s_dwLSThread = uDummy;
		}

		CloseHandle(hLitestepEvent);
	}

	if(NULL == g_hLitestepThread)
	{
		s_bFailLoad = TRUE;
	}

	return;
}


/*--------UnloadStartupItems----------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID UnloadStartupItems(
	VOID
)
{
	/*--------------------------------------------------------------------------
	 Verify Startup Thread has finished
	--------------------------------------------------------------------------*/
	if(NULL != g_hStartupThread)
	{
		DWORD dwRet;

		/*----------------------------------------------------------------------
		 Allow for a long 10 second timeout before killing the thread.
		 Hopefully it is never actually necessary. Also, lets tell the user to
		 report this, and then check again to see if the thread finally ended so
		 we don't kill it unnecessarily.
		----------------------------------------------------------------------*/
		dwRet = WaitForSingleObject(g_hStartupThread, 10000);

		if(WAIT_OBJECT_0 != dwRet)
		{
			WarningMessageBox(
				 PLSTEXT_TERMINATETHREAD_STARTUP_MSG
				,PLSTEXT_TERMINATETHREAD_CAPTION
				,MB_OK
			);

			dwRet = WaitForSingleObject(g_hStartupThread, 0);

			if(WAIT_OBJECT_0 != dwRet)
			{
				TerminateThread(g_hStartupThread, (DWORD)-1);
			}
		}

		CloseHandle(g_hStartupThread);
		g_hStartupThread = NULL;
	}

	return;
}

/*--------LoadStartupItems------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID LoadStartupItems(
	VOID
)
{
	/*--------------------------------------------------------------------------
	 If SHIFT key is being held down, disable startup items
	--------------------------------------------------------------------------*/
	if(dSTARTUP_DISABLE != g_dwStartup && !(GetAsyncKeyState(VK_SHIFT)&0x8000))
	{
		SS_RunStartup fpStartup;

		if(dSTARTUP_FORCE == g_dwStartup)
		{
			fpStartup = (SS_RunStartup)GetProcAddress(
				 s_hShellsvc
				,"SS_ForceStartup"
			);
		}
		else
		{
			fpStartup = (SS_RunStartup)GetProcAddress(
				 s_hShellsvc
				,"SS_RunStartup"
			);
		}

		if(NULL != fpStartup)
		{
			HANDLE hStartupEvent;

			hStartupEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

			if(NULL != hStartupEvent)
			{
				UINT uDummy;

				g_hStartupThread = (HANDLE)_beginthreadex(
					 NULL
					,0
					,fpStartup
					,hStartupEvent
					,0
					,&uDummy
				);

				if(NULL != g_hStartupThread)
				{
					WaitForSingleObject(hStartupEvent, INFINITE);
					s_dwSUThread = uDummy; // Currently unused
				}

				CloseHandle(hStartupEvent);
			}
		}
	}

	return;
}


/*--------UnloadShellServices---------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID UnloadShellServices(
	VOID
)
{
	/*--------------------------------------------------------------------------
	 Verify Services Thread has finished
	--------------------------------------------------------------------------*/
	if(NULL != g_hServicesThread)
	{
		DWORD dwRet;

		/*----------------------------------------------------------------------
		 Exit Shell Service thread
		----------------------------------------------------------------------*/
		PostThreadMessage(s_dwSSThread, PM_QUIT, 0, 0);

		/*----------------------------------------------------------------------
		 Allow for a long 10 second timeout but nastily killing the thread.
		 Hopefully it is never actually necessary. Also, lets tell the user to
		 report this, and then check again to see if the thread finally ended so
		 we don't kill it unnecessarily.
		----------------------------------------------------------------------*/
		dwRet = WaitForSingleObject(g_hServicesThread, 10000);

		if(WAIT_OBJECT_0 != dwRet)
		{
			WarningMessageBox(
				 PLSTEXT_TERMINATETHREAD_SERVICE_MSG
				,PLSTEXT_TERMINATETHREAD_CAPTION
				,MB_OK
			);

			dwRet = WaitForSingleObject(g_hServicesThread, 0);

			if(WAIT_OBJECT_0 != dwRet)
			{
				TerminateThread(g_hServicesThread, (DWORD)-1);
			}
		}

		CloseHandle(g_hServicesThread);
		g_hServicesThread = NULL;
	}

	return;
}

/*--------LoadShellServices-----------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID LoadShellServices(
	VOID
)
{
	HANDLE hServicesEvent;

	hServicesEvent = CreateEvent(NULL, TRUE, FALSE, NULL);

	if(NULL != hServicesEvent)
	{
		UINT uDummy;

		g_hServicesThread = (HANDLE)_beginthreadex(
			 NULL
			,0
			,RunShellServices
			,hServicesEvent
			,0
			,&uDummy
		);

		if(NULL != g_hServicesThread)
		{
			WaitForSingleObject(hServicesEvent, INFINITE);
			s_dwSSThread = uDummy;
		}

		CloseHandle(hServicesEvent);
	}

	return;
}


/*--------UnloadConfig----------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID UnloadConfig(
	VOID
)
{
	if(NULL != s_hConfig)
	{
		SC_FreeBuffer fpFreeBuffer;

		fpFreeBuffer = (SC_FreeBuffer)GetProcAddress(
			 s_hConfig
			,"SH_CFG_FreeBuffer"
		);

		if(NULL != fpFreeBuffer)
		{
			fpFreeBuffer();
		}

		FreeLibrary( s_hConfig );
		s_hConfig = NULL;
	}

	return;
}


/*--------LoadConfig------------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID LoadConfig(
	VOID
)
{
	_TCHAR szPath[MAX_PATH];

	_stprintf(szPath, _T("%s\\%s"), g_pszAppPath, _T("config.dll"));
	s_hConfig = LoadLibrary(szPath);

	if(NULL != s_hConfig)
	{
		SC_FillBuffer fpFillBuffer;

		fpFillBuffer = (SC_FillBuffer)GetProcAddress(
			 s_hConfig
			,"SH_CFG_FillBuffer"
		);

		if(NULL != fpFillBuffer)
		{
			fpFillBuffer(g_pszAppPath, g_pszCfgFile);
		}
	}

	return;
}


/*--------UnloadServices--------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID UnloadServices(
	VOID
)
{
	if(NULL != s_hShellsvc)
	{
		UnloadStartupItems();
		UnloadShellServices();

		FreeLibrary(s_hShellsvc);
		s_hShellsvc = NULL;
	}

#ifdef PLS_EXCEPTION_HANDLER

	/*--------------------------------------------------------------------------
	 Unload Exception handler
	--------------------------------------------------------------------------*/
	PLS_UnloadExceptionHandler();

#endif /* PLS_EXCEPTION_HANDLER */

	return;
}


/*--------LoadServices----------------------------------------------------------
 Purpose:
 - 
------------------------------------------------------------------------------*/
VOID LoadServices(
	VOID
)
{
#ifdef PLS_EXCEPTION_HANDLER

	/*--------------------------------------------------------------------------
	 Load Exception handler
	--------------------------------------------------------------------------*/
	if(!PLS_LoadExceptionHandler())
	{
		OutputDebugString(_T("Couldn't load the exception handler\r\n"));
	}

#endif /* PLS_EXCEPTION_HANDLER */

	if(g_bServices)
	{
		_TCHAR szPath[MAX_PATH];

		_stprintf(szPath, _T("%s\\%s"), g_pszAppPath, _T("shellsvc.dll"));
		s_hShellsvc = LoadLibrary(szPath);

		if(NULL != s_hShellsvc)
		{
			/*------------------------------------------------------------------
			 Load Startup Items Thread
			------------------------------------------------------------------*/
			LoadStartupItems();

			/*------------------------------------------------------------------
			 Load Shell Services [NotifyIcon/Shell(Task)Hook] Thread
			------------------------------------------------------------------*/
			LoadShellServices();
		}
	}

	return;
}

/*--------End Of File---------------------------------------------------------*/
