/*----------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2002 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
----------------------------------------------------------------------*/


#ifndef _INC_EXCEPT
#define _INC_EXCEPT

//#define PLS_EXCEPTION_HANDLER
#ifdef PLS_EXCEPTION_HANDLER

/*----------------------------------------------------------------------
 This is the information stored for each stack frame in a stack trace
----------------------------------------------------------------------*/
typedef struct _PLS_STACKFRAME
{
	ULONG	ulFramePointer;		/* The frame pointer [ebp] */
	ULONG	ulStackPointer;		/* The stack pointer [esp] */
	ULONG	ulFunctionAddress;	/* Address of the current function */
	ULONG	ulReturnAddress;	/* Marks the address which the function
	      	                	   will return to when done. */
	HMODULE	hModule;			/* Module handle for the module which
	       	        			   contains current function. */
	LPCTSTR	szFunctionName;		/* The name of the function */
} PLS_STACKFRAME, *PPLS_STACKFRAME;


/*----------------------------------------------------------------------
 This is the structure of the stacktrace that is exposed by the
 exception handling mechanism. The stacktrace is stored like an
 ordinary linked list, with the pOuterLevel set to NULL for the
 outermost frame in the trace.
----------------------------------------------------------------------*/
typedef struct _PLS_STACKTRACE
{
	PLS_STACKFRAME			StackFrame;
	struct _PLS_STACKTRACE*	pOuterLevel;
} PLS_STACKTRACE, *PPLS_STACKTRACE;


/*----------------------------------------------------------------------
 This defines how an exception handler for purels should look like.
----------------------------------------------------------------------*/
typedef BOOL (CALLBACK *PLS_ExceptionHandler)(
	EXCEPTION_POINTERS*	pExceptionData,
	PPLS_STACKTRACE		pStackTrace
);

#endif /* PLS_EXCEPTION_HANDLER */


#endif /* _INC_EXCEPT */
