/*----------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
	USA
----------------------------------------------------------------------*/


#ifndef _INC_LSAPI
#define _INC_LSAPI

#ifndef NOLSMSGS
#include "lsmsgs.h"
#endif

#ifndef NOLSMULTIMON
#include "lsmultimon.h"
#endif

#include <tchar.h> /* TCHAR defines */
#include <stdio.h> /* Included for FILE typedef */

//#define magicDWord 0x49474541
#define magicDWord magicDWord
static const long magicDWord = 0x49474541;

#define MAX_LINE_LENGTH 4096

#ifndef LSAPI
#define LSAPI __declspec( dllimport )
#endif

#ifdef	__cplusplus
	extern "C" {
#endif	/* __cplusplus */

		/* File Access */
	LSAPI FILE* LCOpen( LPCTSTR );
	LSAPI BOOL LCClose( FILE* );
	LSAPI BOOL LCReadNextCommand( FILE*, LPTSTR, DWORD );
	LSAPI BOOL LCReadNextConfig( FILE*, LPCTSTR, LPTSTR, DWORD );
	LSAPI BOOL LCReadNextLine( FILE*, LPTSTR, DWORD );

		/* Configuration Buffer Setup */
	LSAPI BOOL SetupRC( LPCTSTR );
	LSAPI VOID CloseRC( VOID );

		/* Settings Retrieval */
	LSAPI INT GetRCInt( LPCTSTR, INT );
	LSAPI BOOL GetRCString( LPCTSTR, LPTSTR, LPCTSTR, size_t );
	LSAPI BOOL GetRCBool( LPCTSTR, BOOL );
	LSAPI BOOL GetRCBoolDef( LPCTSTR, BOOL );
	LSAPI BOOL GetRCLine( LPCTSTR, LPTSTR, size_t, LPCTSTR );
	LSAPI COLORREF GetRCColor( LPCTSTR, COLORREF );

	LSAPI BOOL LSGetVariable( LPCTSTR, LPTSTR );
	LSAPI BOOL LSGetVariableEx( LPCTSTR, LPTSTR, DWORD );
	LSAPI VOID LSSetVariable( LPCTSTR, LPCTSTR );

		/* Bang Command Control */
	typedef VOID (__cdecl *FP_BANG_COMMAND)(HWND, LPCTSTR);
	typedef VOID (__cdecl *FP_BANG_COMMAND_EX)(HWND, LPCTSTR, LPCTSTR);
	LSAPI BOOL IsBangCommand( LPCTSTR );
	LSAPI BOOL AddBangCommand( LPCTSTR, FP_BANG_COMMAND );
	LSAPI BOOL AddBangCommandEx( LPCTSTR, FP_BANG_COMMAND_EX );
	LSAPI BOOL RemoveBangCommand( LPCTSTR );
	LSAPI BOOL ParseBangCommand( HWND, LPCTSTR, LPCTSTR );

		/* String Buffer Parsing */
	LSAPI VOID VarExpansion( LPTSTR, LPCTSTR );
	LSAPI VOID VarExpansionEx( LPTSTR, LPCTSTR, DWORD );
	LSAPI INT LCTokenize( LPCTSTR, LPTSTR*, DWORD, LPTSTR );
	LSAPI INT CommandTokenize( LPCTSTR, LPTSTR*, DWORD, LPTSTR );
	LSAPI VOID CommandParse( LPCTSTR, LPTSTR, LPTSTR, DWORD, DWORD );
	LSAPI BOOL GetToken( LPCTSTR, LPTSTR, LPCTSTR*, BOOL );

		/* Pattern Matching */
	LSAPI BOOL match( LPCTSTR, LPCTSTR );
	LSAPI INT matche( LPCTSTR, LPCTSTR );
	LSAPI BOOL is_pattern( LPCTSTR );
	LSAPI BOOL is_valid_pattern( LPCTSTR, PINT );

		/* Command Execution */
	LSAPI HINSTANCE LSExecute( HWND, LPCTSTR, INT );
	LSAPI HINSTANCE LSExecuteEx( HWND, LPCTSTR, LPCTSTR, LPCTSTR, LPCTSTR, INT );

		/* Shell Utilities */
	LSAPI HWND GetLitestepWnd( VOID );
	LSAPI BOOL WINAPI LSGetLitestepPath( LPTSTR, size_t );
	LSAPI BOOL WINAPI LSGetImagePath( LPTSTR, size_t );
	LSAPI VOID SetDesktopArea( INT, INT, INT, INT );

	LSAPI BOOL WINAPI LSLog( INT, LPCTSTR, LPCTSTR );
	LSAPI BOOL WINAPIV LSLogPrintf( INT, LPCTSTR, LPCTSTR, ...);
	LSAPI INT GetRCCoordinate(LPCTSTR, INT, INT);
	LSAPI INT ParseCoordinate(LPCTSTR, INT, INT);

		/* Graphics Control */
	LSAPI HRGN BitmapToRegion( HBITMAP, COLORREF, COLORREF, INT, INT );
	LSAPI HBITMAP BitmapFromIcon( HICON );
	LSAPI HBITMAP LoadLSImage( LPCTSTR, LPCTSTR );
	LSAPI HICON LoadLSIcon( LPCTSTR, LPCTSTR );
	LSAPI VOID GetLSBitmapSize( HBITMAP, INT*, INT* );
	LSAPI VOID TransparentBltLS( HDC, INT, INT, INT, INT, HDC, INT, INT, COLORREF );
	LSAPI VOID Frame3D( HDC, RECT, COLORREF, COLORREF, INT );

#ifndef NOLSMULTIMON
		/* Multi-Monitor Support */
	LSAPI INT LSGetSystemMetrics( INT );
	LSAPI HMONITOR LSMonitorFromWindow( HWND, DWORD );
	LSAPI HMONITOR LSMonitorFromRect( LPCRECT, DWORD );
	LSAPI HMONITOR LSMonitorFromPoint( POINT, DWORD );
	LSAPI BOOL LSGetMonitorInfo( HMONITOR, LPMONITORINFO );
	LSAPI BOOL LSEnumDisplayMonitors( HDC, LPCRECT, MONITORENUMPROC, LPARAM );
	LSAPI BOOL LSEnumDisplayDevices( PVOID, DWORD, PDISPLAY_DEVICE, DWORD );
#endif /* NOLSMULTIMON */

		/* Misc */
	LSAPI VOID GetResStr( HINSTANCE, UINT, LPTSTR, INT, LPCTSTR );
	LSAPI VOID GetResStrEx( HINSTANCE, UINT, LPTSTR, INT, LPCTSTR, ... );

#ifdef	__cplusplus
	};
#endif	/* __cplusplus */

#undef LSAPI

#endif /* _INC_LSAPI */
