/*------------------------------------------------------------------------------
	This is part of the PureLS Source code.

	Copyright (C) 2000-2003 The PureLS Development Team

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the Free
    Software Foundation; either version 2 of the License, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
    more details.

    You should have received a copy of the GNU General Public License along with
    this program; if not, write to the Free Software Foundation, Inc.,
    59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 Lets just make sure we aren't depending on IE
------------------------------------------------------------------------------*/
#define _WIN32_IE 0x0000


/*---System Includes----------------------------------------------------------*/
#include <tchar.h>
#include <windows.h>
#include <shellapi.h>
/*---Shell Includes-----------------------------------------------------------*/
#include "shellsvc.h"
#include "../sdk/include/purels.h"
/*----------------------------------------------------------------------------*/


/*---Global Variables---------------------------------------------------------*/
/*----------------------------------------------------------------------------*/

#ifdef LOG_APPBAR
extern FILE * fAppBarLog;

static void _logAppbar(PCSHELLAPPBARDATA const psad )
{
	if(NULL == fAppBarLog)
	{
		return;
	}

	_ftprintf(fAppBarLog, _T("============\n"));
	_ftprintf(fAppBarLog, _T("=AppBarMsg:=\n"));
	_ftprintf(fAppBarLog, _T("================================================================================\n"));

	_ftprintf(fAppBarLog, _T("dwMessage:"));
	switch(psad->dwMessage)
	{
	case ABM_ACTIVATE:
		_ftprintf(fAppBarLog, _T(" ABM_ACTIVATE"));
		break;
	case ABM_GETAUTOHIDEBAR:
		_ftprintf(fAppBarLog, _T(" ABM_GETAUTOHIDEBAR"));
		break;
	case ABM_GETSTATE:
		_ftprintf(fAppBarLog, _T(" ABM_GETSTATE"));
		break;
	case ABM_GETTASKBARPOS:
		_ftprintf(fAppBarLog, _T(" ABM_GETTASKBARPOS"));
		break;
	case ABM_NEW:
		_ftprintf(fAppBarLog, _T(" ABM_NEW"));
		break;
	case ABM_QUERYPOS:
		_ftprintf(fAppBarLog, _T(" ABM_QUERYPOS"));
		break;
	case ABM_REMOVE:
		_ftprintf(fAppBarLog, _T(" ABM_REMOVE"));
		break;
	case ABM_SETAUTOHIDEBAR:
		_ftprintf(fAppBarLog, _T(" ABM_SETAUTOHIDEBAR"));
		break;
	case ABM_SETPOS:
		_ftprintf(fAppBarLog, _T(" ABM_SETPOS"));
		break;
	case ABM_SETSTATE:
		_ftprintf(fAppBarLog, _T(" ABM_SETSTATE"));
		break;
	case ABM_WINDOWPOSCHANGED:
		_ftprintf(fAppBarLog, _T(" ABM_WINDOWPOSCHANGED"));
		break;
	}
	_ftprintf(fAppBarLog, _T("\n"));

	_ftprintf(fAppBarLog, _T("================================================================================\n"));
	
	_ftprintf(fAppBarLog, _T("SHELLAPPBARDATA\n{\n"));
	_ftprintf(fAppBarLog, _T("\tAPPBARDATA (0x%08X)\n\t{\n"), &psad->abd);

	_ftprintf(fAppBarLog, _T("\t\tDWORD  cbSize           = 0x%08X\n"), psad->abd.cbSize);
	_ftprintf(fAppBarLog, _T("\t\tHWND   hWnd             = 0x%08X\n"), psad->abd.hWnd);
	_ftprintf(fAppBarLog, _T("\t\tUINT   uCallbackMessage = 0x%08X\n"), psad->abd.uCallbackMessage);
	_ftprintf(fAppBarLog, _T("\t\tUINT   uEdge            = 0x%08X\n"), psad->abd.uEdge);

	_ftprintf(fAppBarLog, _T("\t\t\t("));
	switch(psad->abd.uEdge)
	{
	case ABE_BOTTOM:
		_ftprintf(fAppBarLog, _T(" ABE_BOTTOM"));
		break;
	case ABE_LEFT:
		_ftprintf(fAppBarLog, _T(" ABE_LEFT"));
		break;
	case ABE_RIGHT:
		_ftprintf(fAppBarLog, _T(" ABE_RIGHT"));
		break;
	case ABE_TOP:
		_ftprintf(fAppBarLog, _T(" ABE_TOP"));
		break;
	default:
		break;
	}

	_ftprintf(fAppBarLog, _T(")\n"));

	_ftprintf(fAppBarLog, _T("\t\tRECT   rc               = 0x%08X\n"), psad->abd.rc);

	_ftprintf(fAppBarLog, _T("\t\t\t(%d,%d,%d,%d)\n"), psad->abd.rc.left, psad->abd.rc.top, psad->abd.rc.right, psad->abd.rc.bottom);

	_ftprintf(fAppBarLog, _T("\t\tLPARAM lParam           = 0x%08X\n"), psad->abd.lParam);

	_ftprintf(fAppBarLog, _T("\t}\n"));

	_ftprintf(fAppBarLog, _T("\tDWORD dwMessage = 0x%08X\n"), psad->dwMessage);
	_ftprintf(fAppBarLog, _T("\tDWORD dwUnknown = 0x%08X\n"), psad->dwUnknown);
	_ftprintf(fAppBarLog, _T("\tDWORD dwNoClue  = 0x%08X\n"), psad->dwNoClue);

	_ftprintf(fAppBarLog, _T("}\n"));
	_ftprintf(fAppBarLog, _T("================================================================================\n\n"));

	fflush(fAppBarLog);

	return;
}
#endif /* LOG_APPBAR */

/*--------onAppbarMsg-----------------------------------------------------------
 Purpose:
 -
------------------------------------------------------------------------------*/
LRESULT onAppbarMsg( PCSHELLAPPBARDATA const psad )
{
	LRESULT lResult = 0;

#ifdef LOG_APPBAR
	_logAppbar(psad);
#endif

	switch(psad->dwMessage)
	{
	case ABM_ACTIVATE:
		lResult = 1;
		break;

	case ABM_GETAUTOHIDEBAR:
		break;

	case ABM_GETSTATE:
		lResult = ABS_ALWAYSONTOP;//|ABS_AUTOHIDE;
		break;

	case ABM_GETTASKBARPOS:
		break;

	case ABM_NEW:
		break;

	case ABM_QUERYPOS:
		lResult = 1;
		break;

	case ABM_REMOVE:
		lResult = 1;
		break;

	case ABM_SETAUTOHIDEBAR:
		break;

	case ABM_SETPOS:
		lResult = 1;
		break;

	case ABM_SETSTATE:
		lResult = 1;
		break;

	case ABM_WINDOWPOSCHANGED:
		lResult = 1;
		break;

	default:
		break;
	}

	return(lResult);
}

/*--------End Of File---------------------------------------------------------*/
